(function ($) {
  /**
   * @namespace
   */
  window.jp = window.jp || {};
  /**
   * @memberOf jp
   */
  jp.exblog = jp.exblog || {};
  /**
   * @namespace service
   * @memberOf jp.exblog
   */
  jp.exblog.service = jp.exblog.service || {};


  /**
   * @memberOf jp.exblog.service
   */
  jp.exblog.service.Profile = (function () {
    /**
     * @param args
     * @constructor
     */
    function Profile(args) {
      //
    }


    /** @enum {number} */
    Profile.LOG_LEVELS = {
      DEBUG: 0, INFO: 1, WARN: 2, ERROR: 9
    };
    /** @type {number} */
    Profile.LOG_LEVEL = Profile.LOG_LEVELS.INFO;
    /** @type {string} */
    Profile.SELECTOR = '.exblog-like-button';
    /** @type {string} */
    Profile.PREFIX = 'xbg_like';
    /** @type {string} */
    Profile.CSS_URL = 'https://image.excite.co.jp/jp/exblog/api/css/exblog.profile.css';
    /** @type {string} */
    Profile.LOGIN_URL = 'https://www.exblog.jp/';
    /** @type {object} */
    Profile.API_CONFIG = {
      SCHEME: 'https',
      HOST: 'api.exblog.jp',
      BASE_PATH: '/profile/',
      PATH: {
        STATUSES: 'like/status/',
        LIKE: 'like/create/',
        UNLIKE: 'like/destroy/',
        DELIKE: 'like/destroy/admin/',
        HISTORY: 'like/history/'
      }
    };
    /** @type {string} */
    Profile.LIKE_STRING = 'Like';

    /**
     * @param var_args
     */
    Profile.debug = function debug(var_args) {
      var args = Array.prototype.slice.call(arguments);
      args.unshift(Profile.LOG_LEVELS.DEBUG);
      Profile.log.apply(this, args);
    };

    /**
     * @param var_args
     */
    Profile.info = function info(var_args) {
      var args = Array.prototype.slice.call(arguments);
      args.unshift(Profile.LOG_LEVELS.INFO);
      Profile.log.apply(this, args);
    };
    /**
     * @param var_args
     */
    Profile.warn = function warn(var_args) {
      var args = Array.prototype.slice.call(arguments);
      args.unshift(Profile.LOG_LEVELS.WARN);
      Profile.log.apply(this, args);
    };
    /**
     * @param var_args
     */
    Profile.error = function error(var_args) {
      var args = Array.prototype.slice.call(arguments);
      args.unshift(Profile.LOG_LEVELS.ERROR);
      Profile.log.apply(this, args);
    };
    /**
     * @param {number} level
     * @param var_args
     */
    Profile.log = function log(level, var_args) {
      level = arguments[0] || Profile.LOG_LEVELS.DEBUG;
      var args = Array.prototype.slice.call(arguments);
      args.shift();

      var f;
      switch (level) {
        case Profile.LOG_LEVELS.ERROR:
          f = 'error';
          break;
        case Profile.LOG_LEVELS.WARN:
          f = 'warn';
          break;
        case Profile.LOG_LEVELS.INFO:
          f = 'info';
          break;
        case Profile.LOG_LEVELS.DEBUG:
        default:
          f = 'debug';
          break;
      }

      if (typeof(console) === 'undefined') return;

      if (console && console[f] && level >= Profile.LOG_LEVEL) {
        if (typeof(console[f]) === 'object') {
          args.unshift(console);
          Function.prototype.call.apply(console[f], args);
        } else {
          console[f].apply(console, args);
        }
      }
    };
    /**
     *
     */
    Profile.logCurrentMethod = function logCurrentMethod() {
      var caller = arguments.callee.caller;
      var args = {};
      for (var i = 0; i < caller.arguments.length; i++) {
        args[i] = caller.arguments[i];
      }
      if (caller.arguments.length == 0) {
        Profile.debug.apply(this, ['+Profile->%s()', Profile.getFunctionName(caller)]);
      } else {
        Profile.debug.apply(this, ['+Profile->%s(%o)', Profile.getFunctionName(caller), args]);
      }
    };
    /**
     * @param {function} f
     * @returns {string}
     */
    Profile.getFunctionName = function getFunctionName(f) {
      var matches = f.toString().match(/^function\s(.+)\(/i);
      if (matches) {
        return matches[1];
      }
      return null;
    };


    /**
     * @param {string} mode
     * @returns {string}
     */
    Profile.getApiUrl = function getApiUrl(mode) {
      return Profile.API_CONFIG.SCHEME + '://'
        + Profile.API_CONFIG.HOST
        + Profile.API_CONFIG.BASE_PATH
        + Profile.API_CONFIG.PATH[mode.toUpperCase()];
    };
    /**
     * @param {object} args
     */
    Profile.callApi = function callApi(args) {
      var success = args.success;
      var error = args.error;

      var defaults = {
        method: 'GET',
        dataType: 'jsonp',
        success: function (data, dataType) {
          if (data.success) {
            success.apply(Profile, arguments);
            return;
          }
          Profile.warn('Call Api Failed %o', arguments);
          error.apply(Profile, arguments);
        }
      };

      delete args.success;
      delete args.error;

      args = $.extend({}, defaults, args);

      try {
        $.ajax(args);
      } catch (e) {
        Profile.error(e);
      }
    };
    /**
     * @param {number} n
     * @returns {string}
     */
    Profile.convertHumanReadableNumber = function convertHumanReadableNumber(n) {
      if (!(n + '').match(/^[0-9]+$/i)) {
        Profile.warn('Invalid Number Format %o', n);
      }
      var units = ['', 'K', 'M', 'G', 'T', 'P'];
      var mod = 1000;
      for (var i = 0; n >= mod && i < units.length - 1; i++) {
        n /= mod;
      }
      n = Math.floor(n * 10) / 10;
      if (n >= 10) {
        n = Math.floor(n);
      }
      return n + units[i];
    };
    /**
     * @param {{message:string, code:Number, error_code:Number}} error
     */
    Profile.handleError = function handleError(error) {
      switch (error.error_code) {
        case 401:
          if (confirm('エキサイトブログプロフィールを利用するには「新規登録」する必要があります。')) {
            location.href = Profile.LOGIN_URL;
          }
          break;
        case 402:
          if (confirm('エキサイトブログプロフィールを利用するには「ログイン」する必要があります。')) {
            location.href = Profile.LOGIN_URL;
          }
          break;
        case 403:
        case 404:
        case 405:
        case 406:
        case 407:
        case 408:
          alert('不正なアクセスです。');
          break;
        case 409:
          alert('イイネ対象のページが既に存在しません。ページを更新してください。');
          break;
        default:
          alert('エラーが発生しました。しばらくしてから再度お試しください。');
          break;
      }
    };


    /**
     * @param {Array} urls
     * @param {string} blogid
     * @param {function} callback
     */
    Profile.getLikeStatuses = function getLikeStatus(urls, blogid, callback) {
      Profile.logCurrentMethod();

      if (!urls.length) {
        callback();
        return;
      }

      Profile.callApi({
        url: Profile.getApiUrl('statuses'),
        data: {
          urls: urls.join('\t'),
          blogid: blogid,
          format: 'jsonp'
        },
        success: function (data, dataType) {
          // array to hash
          var urls = {};
          for (var i = 0; i < data.items.length; i++) {
            var item = data.items[i];
            urls[item.url] = item;
          }
          callback(urls);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {
          callback();
        }
      });
    };
    /**
     * @param {string} url
     * @param {function} callback
     */
    Profile.putLike = function putLike(url, callback) {
      Profile.logCurrentMethod();

      Profile.callApi({
        url: Profile.getApiUrl('like'),
        data: {
          url: url,
          format: 'jsonp'
        },
        success: function (data, dataType) {
          callback(data.item);
        },
        error: function (data, dataType) {
          callback();
          Profile.handleError(data.error);
        }
      });
    };
    /**
     * @param {string} url
     * @param {function} callback
     */
    Profile.putUnlike = function putUnlike(url, callback) {
      Profile.logCurrentMethod();

      Profile.callApi({
        url: Profile.getApiUrl('unlike'),
        data: {
          url: url,
          format: 'jsonp'
        },
        success: function (data, dataType) {
          callback(data.item);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {
          callback();
          Profile.handleError(data.error);
        }
      });
    };
    /**
     * @param {string} url
     * @param {string} blogid
     * @param {function} callback
     */
    Profile.putDelike = function putDelike(url, blogid, callback) {
      Profile.logCurrentMethod();

      Profile.callApi({
        url: Profile.getApiUrl('delike'),
        data: {
          url: url,
          from_id: blogid,
          format: 'jsonp'
        },
        success: function (data, dataType) {
          callback(data.item);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {
          callback();
          Profile.handleError(data.error);
        }
      });
    };


    /**
     * @param {string} blogurl
     * @param {string} url
     * @param {function} callback
     */
    Profile.getLikeHistory = function getLikeHistory(blogurl, url, callback) {
      Profile.logCurrentMethod();

      Profile.callApi({
        url: Profile.getApiUrl('history'),
        data: {
          blogurl: blogurl,
          url: url,
          page: 1,
          confirm: 0,
          format: 'jsonp'
        },
        success: function (data, dataType) {
          callback(data.items);
        },
        error: function (data, dataType) {
          callback();
          Profile.handleError(data.error);
        }
      });
    };


    /**
     * @param {jQuery} element
     * @param {{liked: boolean, count: number}} data
     */
    Profile.renderLikeButton = function renderLikeButton(element, data) {
      Profile.logCurrentMethod();

      if (!data) return;

      var cls = data.liked ? Profile.PREFIX + '_li_yet' : Profile.PREFIX + '_li';
      var click = data.liked ? Profile.unlike : Profile.like;
      var dataurl = data.url;
      var url = dataurl.split('exblog.jp');
      var srl = url[1];
      var blogurl = url[0].replace('http://', '').replace('https://', '');
      blogurl = blogurl.replace('.', '');

      var span;
      if (data.count == 0) {
        span = $('<span>').addClass(Profile.PREFIX + '_lc').append($('<span>').addClass(Profile.PREFIX + '_number').text(Profile.convertHumanReadableNumber(data.count)));
      } else if (ExBlog.Agent.isSmartphone()) {
        span = $('<span>').addClass(Profile.PREFIX + '_lc').click(function () {
          ga('send', 'event', 'LikeHistoryButton', 'click', 'sp');
          // blogmate対応
          if (srl.match(/community/)) {
            Profile.getLikeHistory(blogurl, dataurl, function (data) {
              if (!data) return;
              window.location.href = 'https://' + Profile.escapeHtml(data[0]['blogurl']) + '.exblog.jp/likes/?blogurl=' + Profile.escapeHtml(data[0]['blogurl']) + '&url=' + dataurl + '&page=1';
            });
            return;
          }
          var likesurl = 'https://' + Profile.escapeHtml(blogurl) + '.exblog.jp/likes/?blogurl=' + Profile.escapeHtml(blogurl) + '&url=' + dataurl + '&page=1';
          if (location.host == 'blog.excite.co.jp') {
            likesurl = 'https://blog.excite.co.jp/' + Profile.escapeHtml(blogurl) + '/likes/?blogurl' + Profile.escapeHtml(blogurl) + '&url=' + dataurl + '&page=1';
          }
          window.location.href = likesurl;
        }).append($('<span>').addClass(Profile.PREFIX + '_number').text(Profile.convertHumanReadableNumber(data.count)))
      } else {
        span = $('<span>').addClass(Profile.PREFIX + '_lc').attr('data-tgt', Profile.PREFIX + '_wd').click(function () {
          $('.' + Profile.PREFIX + '_modal').remove();
          $('body').append($(
            '<div class="' + Profile.PREFIX + '_modal ' + Profile.PREFIX + '_wd">'
            + '  <div class="' + Profile.PREFIX + '_modalBody">'
            + '    <div id="' + Profile.PREFIX + '_like_num"></div>'
            + '    <div class="' + Profile.PREFIX + '_close">×close</div>'
            + '    <table id="' + Profile.PREFIX + '_like_table"></table>'
            + '    <div id="' + Profile.PREFIX + '_like_page"></div>'
            + '    <div id="' + Profile.PREFIX + '_like_more"></div>'
            + '  </div>'
            + '  <div class="' + Profile.PREFIX + '_modalBK"></div>'
            + '</div>'));
          var wn = '.' + $(this).attr('data-tgt');
          var modal = $(wn).find('.' + Profile.PREFIX + '_modalBody');
          var mW = modal.innerWidth() / 2;
          var mH = modal.innerHeight() / 2;
          modal.css({'margin-left': -mW, 'margin-top': -mH});
          $(wn).fadeIn(500);
          var div1 = $('#' + Profile.PREFIX + '_like_num').empty();
          var div2 = $('#' + Profile.PREFIX + '_like_page').empty();
          var div3 = $('#' + Profile.PREFIX + '_like_more').empty();
          var table = $('#' + Profile.PREFIX + '_like_table').empty();
          var tbody = $('<tbody>');
          Profile.getLikeHistory(blogurl, dataurl, function (data) {
            if (!data) return;
            for (var i = 0; i < data.length; i++) {
              var value = data[i];
              if (i == 0) {
                div1.text(value.cnt + '人がイイネしています');
                var to = value.cnt;
                if (value.to_num < to) {
                  to = value.to_num;
                }
                div2.text(value.cnt + '人中' + value.from_num + '-' + to + '件を表示');
                var likesurl = 'https://' + Profile.escapeHtml(value.blogurl) + '.exblog.jp/likes/?blogurl=' + Profile.escapeHtml(value.blogurl) + '&url=' + dataurl + '&page=1';
                if (location.host == 'blog.excite.co.jp') {
                  likesurl = 'https://blog.excite.co.jp/' + Profile.escapeHtml(value.blogurl) + '/likes/?blogurl' + Profile.escapeHtml(value.blogurl) + '&url=' + dataurl + '&page=1';
                }
                div3.append($('<a>').attr('href', likesurl).text('もっと見る'));
              }
              var nickname;
              if (!value.blogid) {
                // todo blogidない場合の表示
                nickname = '<td></td>';
              } else if (value.blogid && !value.like_blogurl) {
                nickname = '<td>ユーザーが退会しているため表示できません。</td>';
              } else {
                nickname = '<td><a href="' + value.like_blogurl + '">' + Profile.escapeHtml(value.usernick) + 'さん</a></td>';
              }
              var td = $('<tr>')
                .append('<td><img src=' + Profile.escapeHtml(value.user_logo) + ' width="30"></td>')
                .append(nickname)
                .append('<td align="right">' + Profile.escapeHtml(value.create_date_str) + '</td>');
              tbody.append(td);
            }
            table.append(tbody);
            $('.' + Profile.PREFIX + '_modal .' + Profile.PREFIX + '_close,.' + Profile.PREFIX + '_modalBK').click(function () {
              $(wn).fadeOut(500);
            });
            ga('send', 'event', 'LikeHistoryButton', 'click', 'dialog');
          });
        }).append($('<span>').addClass(Profile.PREFIX + '_number').text(Profile.convertHumanReadableNumber(data.count)))
      }
      var button = $('<div>').addClass(Profile.PREFIX).data(element.data()).append(
        $('<div>').addClass(Profile.PREFIX + '_default').append(
          $('<span>').addClass(cls).append(
            $('<span>').addClass(Profile.PREFIX + '_ll').text(Profile.LIKE_STRING).click(function () {
              click(button);
            })
          ).append(span)
        )
      );
      element.replaceWith(button);
    };

    /**
     * @param {jQuery} element
     */
    Profile.like = function like(element) {
      Profile.logCurrentMethod();

      var url = element.data('url');
      Profile.putLike(url, function (data) {
        Profile.renderLikeButton(element, data);
      });
    };
    /**
     * @param {jQuery} element
     */
    Profile.unlike = function unlike(element) {
      Profile.logCurrentMethod();

      var url = element.data('url');
      Profile.putUnlike(url, function (data) {
        Profile.renderLikeButton(element, data);
      });
    };


    /**
     *
     */
    Profile.init = function init() {
      Profile.logCurrentMethod();

      Profile.loadCss();
      Profile.setupLikeButtons();
    };
    /**
     *
     */
    Profile.loadCss = function loadCss() {
      Profile.logCurrentMethod();

      $('head').append(
        $('<link>').attr({
          rel: 'stylesheet',
          type: 'text/css',
          href: Profile.CSS_URL
        })
      );
    };
    /**
     *
     */
    Profile.setupLikeButtons = function setupLikeButton() {
      Profile.logCurrentMethod();

      var urls = [];
      var blogid = null;
      $(Profile.SELECTOR).each(function () {
        var url = $(this).data('url');
        var bid = $(this).data('blogid');
        urls.push(url);
        if (!blogid && bid) {
          blogid = bid;
        }
      });

      Profile.getLikeStatuses(urls, blogid, function (data) {
        if (!data) return;

        $(Profile.SELECTOR).each(function () {
          var url = $(this).data('url');
          var d = data[url];
          if (d) {
            Profile.renderLikeButton($(this), d);
          } else {
            Profile.warn('Status data is not found %o', url);
          }
        });
      });
    };

    Profile.escapeHtml = function escapeHtml(string) {
      return string
        ? string.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#x27;').replace(/`/g, '&#x60;')
        : '';
    }


    return Profile;
  })();

  /**
   * @type {?object} ExBlog
   */
  var ExBlog = ExBlog || {};
  ExBlog.Agent = (function () {
    /**
     * @constructor
     */
    function Agent() {
    }

    /**
     * @returns {bool}
     */
    Agent.isSmartphone = function () {
      return Agent.isIPhone() || Agent.isAndroid() || Agent.isBlackBerry();
    };
    /**
     * @returns {bool}
     */
    Agent.isIPhone = function () {
      return navigator.userAgent.indexOf('iPhone') >= 0;
    };
    /**
     * @returns {bool}
     */
    Agent.isAndroid = function () {
      var ua = navigator.userAgent;
      return ua.indexOf('Android') >= 0 || ua.indexOf('Mobile') >= 0;
    };
    /**
     * @returns {bool}
     */
    Agent.isBlackBerry = function () {
      return navigator.userAgent.indexOf('BlackBerry') >= 0;
    };
    return Agent;
  })();

  $(function () {
    jp.exblog.service.Profile.debug('!Profile Ready');
    jp.exblog.service.Profile.init();
  });
})(jQuery);
